#!/usr/bin/env python3
# -*- coding: utf-8 -*-
info = """
Author: Mauricio Fernández
ORCID: http://orcid.org/0000-0003-1840-1243
Email: mauricio.fernandez.lb@gmail.com

File history:
    Created on January 21th, 2019
    Edited on March 22th, 2019 - corrected some typos
    Edited on April 12th, 2019 - added weblink and doi

This main file is a supplementary Python 3 script provided for the manuscript
    "On-the-fly adaptivity for nonlinear twoscale simulations using artificial neural networks and reduced order modeling" 
    by Felix Fritzen, Mauricio Fernández and Fredrik Larsson
    web: https://www.frontiersin.org/articles/10.3389/fmats.2019.00075/full
    doi: 10.3389/fmats.2019.00075
    
The main file imports the FOM and ROM data (see manuscript) and calls:
    1) the effective stress surrogate ANN1 of the paper (with corresponding MRNE)
    2) the corresponding quality indicator for the stress surrogate ANN1 of the paper (with corresponding accuracies)
"""
print(info)

import numpy as np
import h5py
import tensorflow as tf
import aux 

#%% Load data

print('\n*********************************************')
print('Load data')

# Load ordered data
data = h5py.File('data/data.h5')

# Print groups in file
print('\nGroups in data file')
for k in data.keys(): print('\t%s' % k)

# Print datasets in each group
print('\nDatasets in each group')
for k in data['training'].keys(): print ('\t%s' % k)

#%% Load effective stress ANN1

print('\n*********************************************')
print('Load and evaluate stress surrogate ANN1 for validation dataset')

# Import validation datasets for the evaluation of the stress surrogate
strain = np.array(data['validation']['strain'])
stress_FE = np.array(data['validation']['stress_FE'])
grad96 = np.array(data['validation']['tangent_ROM96']).reshape([-1,6,6])
data.close()

# Path to ANN stress model
ANN1_path = 'models/stress'

# Start session
config = tf.ConfigProto(allow_soft_placement=True)
sess = tf.Session(config=config)
tf.saved_model.loader.load(sess,['tag'],ANN1_path)

# Get input placeholder and ANN model
graph = tf.get_default_graph()
strain_p = graph.get_tensor_by_name('myxp:0')
ANN = graph.get_tensor_by_name('mymodel:0')

# Gradient function
grad = tf.stack([tf.gradients(ANN[:,i],strain_p)[0] for i in range(ANN.shape[-1])],axis=1)
grad_sym = (grad+tf.transpose(grad,perm=(0,2,1)))/2
grad_skw = (grad-tf.transpose(grad,perm=(0,2,1)))/2

# Evaluate ANN stress model and print mean relative norm error
temp = sess.run(ANN,{strain_p:strain})
MRNE = np.mean(np.linalg.norm(stress_FE - temp,axis=1)/np.linalg.norm(stress_FE,axis=1))
print('Mean relative norm error (MRNE) of stress surrogate for validation data (see Tab.1 in manuscript): \n\t%.4f' % MRNE)

# Evaluate skew part of ANN gradient and print mean relative amount
temp = sess.run(grad_skw,{strain_p:strain})
temp2 = sess.run(grad,{strain_p:strain})
mra_skw = np.mean(np.linalg.norm(temp,axis=(1,2))/np.linalg.norm(temp2,axis=(1,2)))
print('Mean relative amount of skew part of gradient with respect to Frobenius norm for validation data (see section 3.2.1 in manuscript): \n\t%.4f' % mra_skw)

# Evaluate symmetrized gradient and print mean relative error w.r.t. to ROM 96 tangent
temp = sess.run(grad_sym,{strain_p:strain})
mre_sym = np.mean(np.linalg.norm(grad96-temp,axis=(1,2))/np.linalg.norm(grad96,axis=(1,2)))
print('Mean relative error of gradient with respect to Frobenius norm for validation data (see section 3.2.1 in manuscript): \n\t%.4f' % mre_sym)

# Close session
sess.close()
tf.reset_default_graph()

#%% Load quality indicator for ANN1

print('\n*********************************************')
print('Load quality indicator for stress surrogate ANN1')

# Load data from randomly shuffled data in file datar.h5 and concatenate all groups
groups = ['training','validation','Monte Carlo']
strain = np.vstack([aux.getstrain(g) for g in groups])
indicator = np.vstack([aux.getannp(ANN1_path,g,[2,0.02]) for g in groups])

# Extract last 10 percent as new validation dataset
split = np.int(0.9*len(strain))
strain_val = strain[split:]
indicator_val = indicator[split:]

# Load quality indicator ANN
indicator_ANN_path = 'models/indicator'

# Start session
config = tf.ConfigProto(allow_soft_placement=True)
sess = tf.Session(config=config)
tf.saved_model.loader.load(sess,['tag'],indicator_ANN_path)

# Get input placeholder and ANN model
graph = tf.get_default_graph()
strain_p = graph.get_tensor_by_name('myxp:0')
ANN = graph.get_tensor_by_name('mymodel:0')

# Evaluate ANN model and print accuracies
temp = sess.run(tf.sigmoid(ANN)>0.5,{strain_p:strain_val})*1
acc = np.sum(temp==indicator_val)/len(indicator_val)
print('Accuracy of quality indicator for ANN1 (see Tab. 4 in manuscript)')
print('Overall accuracy:\n\t%.4f' % acc)
pos = [i for i in range(len(indicator_val)) if indicator_val[i]==0]
acc0 = np.sum(temp[pos]==0)/len(pos)
print('ACC 0:\n\t%.4f' % acc0)
pos = [i for i in range(len(indicator_val)) if indicator_val[i]==1]
acc1 = np.sum(temp[pos]==1)/len(pos)
print('ACC 1: \n\t%.4f' % acc1)

# Close session
sess.close()
tf.reset_default_graph()